***** Make Friends with SAS, Day 3: Increasing Efficiency with ARRAYS, LOOPS, and MACROS;
***** Created by Lesa Hoffman;

* Output options: all can be turned on/off by adding or removing the "NO":
  page number, date, centering, or page breaks, page length and width; 
OPTIONS nonumber nodate nocenter formdlim='-' pagesize=MAX linesize=120;
TITLE;   * Shut off default SAS titles;


*******************************************************************************
******* 					 	DO LOOPS						        *******
******************************************************************************;

* DO loops can be used in DATA steps to do something repeatedly.
  Here, and example making up make data (i.e., simulation).
  The RAND functions create variables that fit certain distributions;

* Simulate cross-sectional data;
DATA fakepeople;   * No SET statement because data dont exist yet;
	CALL STREAMINIT(8675309); 		* Set random number seed for RAND functions;
	DO PersonID=1 TO 500 BY 1;			* Start making 500 people;
		NormalVar = RAND("NORMAL",0,1); * Make normal variable with mean=0, SD=1;
		CountVar  = RAND("POISSON",3);  * Count variable with mean=var=3;
		UniformVar = RAND("UNIFORM");   * Uniform variable ranging from 0 to 1;
		OUTPUT;						    * Write that row of data with 4 variables;
	END;  * END shuts off DO;
RUN;

* Now lets see what it made;
TITLE1 "Our New Variable Distributions!";
PROC UNIVARIATE NOPRINT DATA=fakepeople;
	VAR NormalVar CountVar UniformVar;
	HISTOGRAM NormalVar CountVar UniformVar;
RUN; TITLE1;

* Simulate longitudinal data with 2 nested DO LOOPS;
DATA longfakepeople;   * No SET statement because data dont exist yet;
	CALL STREAMINIT(8675309); 		* Set random number seed for RAND functions;
	DO PersonID=1 TO 50 BY 1;				* Start making 50 people;
		DO Time=1 TO 5 BY 1;				* Start making 5 occasions;
			NormalVar = RAND("NORMAL",0,1); * Make normal variable with mean=0, SD=1;
			CountVar  = RAND("POISSON",3);  * Count variable with mean=var=3;
			UniformVar = RAND("UNIFORM");   * Uniform variable ranging from 0 to 1;
			OUTPUT;						    * Write that row of data with 4 variables;
		END;  * END shuts off Time DO;
	END;	  * END shuts off Person DO;
RUN;
		


*******************************************************************************
******* 					ARRAYS WITH DO LOOPS						*******
******************************************************************************;

* ARRAY is a short-hand for a list of variables
  ARRAY names must not duplicate any variable names
  Parentheses after array name is #variables, followed by variable list:
  Arraynam(1)=var1, arraynam(2)=var2, etc;

* Here is an example that gets rid of missing values in a series;
DATA &datafile.; SET &datafile.;
    ARRAY anew (100) newvar1 -  newvar100;  * Ordered by name;
    ARRAY aold (100) oldvar1 -- oldvar100;  * Ordered in dataset;
    DO i = 1 TO 100 BY 1;		* DO loop says do what follows 100 times;
		* If old has missing data, change new to system missing;
    	  IF aold(i) = 999 THEN anew(i) = .;
		* Otherwise, copy old value;
          ELSE anew(i) = aold(i);
    END; * END is needed to shut off DO;
	DROP i;  * Counting variable is removed from data;
RUN;

* Your turn;






*******************************************************************************
******* 			MACRO PROGRAMS THAT WRITE SAS CODE					*******
******************************************************************************;

* MACRO options for debugging macro programs;
OPTIONS merror serror mprint mlogic symbolgen spool; 


* So far we have seen macro variables. Now we will see macro programs!
  Macro commands begin with %, such as we have seen so far;
%LET filepath=F:\13_Psyc930SAS\Day3_Array_DO_Macros;
%LET dataname=work.GRI_data;


************************************************************************************;
* Other macro commands can iteratively write SAS code, such as in DO LOOPS;
* Lets write a macro program that adds variable labels to our 24 new variables;
************************************************************************************;





************************************************************************************;
* Now lets write a program that systematically replaces sample sizes
  in our longitudinal fake people, and adds reps per condition;
************************************************************************************;




************************************************************************************;
* Now lets a write a program that systematically imports multiple csv data files
  and merges them all into a single SAS data file;
************************************************************************************;



************************************************************************************;
* Now lets a write a program that systematically imports multiple excel worksheets
  and merges them all into a single SAS data file, adding a subject variable;
************************************************************************************;




